@@ -348,7 +348,7 @@ function howMuchToRead(n, state) {
     return 0;
   if (state.objectMode)
     return 1;
-  if (n !== n) {
+  if (Number.isNaN(n)) {
     // Only flow one buffer at a time
     if (state.flowing && state.length)
       return state.buffer.head.data.length;
@@ -754,7 +754,7 @@ Readable.prototype.unpipe = function(dest) {
     state.flowing = false;
 
     for (var i = 0; i < len; i++)
-      dests[i].emit('unpipe', this, unpipeInfo);
+      dests[i].emit('unpipe', this, { hasUnpiped: false });
     return this;
   }
 
