@@ -307,7 +307,6 @@ Writable.prototype.uncork = function() {
 
     if (!state.writing &&
         !state.corked &&
-        !state.finished &&
         !state.bufferProcessing &&
         state.bufferedRequest)
       clearBuffer(this, state);
@@ -552,7 +551,7 @@ function clearBuffer(stream, state) {
 }
 
 Writable.prototype._write = function(chunk, encoding, cb) {
-  cb(new errors.Error('ERR_METHOD_NOT_IMPLEMENTED', '_transform'));
+  cb(new errors.Error('ERR_METHOD_NOT_IMPLEMENTED', '_write'));
 };
 
 Writable.prototype._writev = null;
@@ -579,7 +578,7 @@ Writable.prototype.end = function(chunk, encoding, cb) {
   }
 
   // ignore unnecessary end() calls.
-  if (!state.ending && !state.finished)
+  if (!state.ending)
     endWritable(this, state, cb);
 };
 
@@ -658,11 +657,9 @@ function onCorkedFinish(corkReq, state, err) {
     cb(err);
     entry = entry.next;
   }
-  if (state.corkedRequestsFree) {
-    state.corkedRequestsFree.next = corkReq;
-  } else {
-    state.corkedRequestsFree = corkReq;
-  }
+
+  // reuse the free corkReq.
+  state.corkedRequestsFree.next = corkReq;
 }
 
 Object.defineProperty(Writable.prototype, 'destroyed', {
