@@ -21,7 +21,6 @@
 
 'use strict';
 
-var domain;
 var spliceOne;
 
 function EventEmitter() {
@@ -34,7 +33,6 @@ EventEmitter.EventEmitter = EventEmitter;
 
 EventEmitter.usingDomains = false;
 
-EventEmitter.prototype.domain = undefined;
 EventEmitter.prototype._events = undefined;
 EventEmitter.prototype._eventsCount = 0;
 EventEmitter.prototype._maxListeners = undefined;
@@ -56,9 +54,7 @@ Object.defineProperty(EventEmitter, 'defaultMaxListeners', {
     return defaultMaxListeners;
   },
   set: function(arg) {
-    // check whether the input is a positive number (whose value is zero or
-    // greater and not a NaN).
-    if (typeof arg !== 'number' || arg < 0 || arg !== arg) {
+    if (typeof arg !== 'number' || arg < 0 || Number.isNaN(arg)) {
       const errors = lazyErrors();
       throw new errors.TypeError('ERR_OUT_OF_RANGE', 'defaultMaxListeners');
     }
@@ -67,14 +63,6 @@ Object.defineProperty(EventEmitter, 'defaultMaxListeners', {
 });
 
 EventEmitter.init = function() {
-  this.domain = null;
-  if (EventEmitter.usingDomains) {
-    // if there is an active domain, then attach to it.
-    domain = domain || require('domain');
-    if (domain.active && !(this instanceof domain.Domain)) {
-      this.domain = domain.active;
-    }
-  }
 
   if (this._events === undefined ||
       this._events === Object.getPrototypeOf(this)._events) {
@@ -88,7 +76,7 @@ EventEmitter.init = function() {
 // Obviously not all Emitters should be limited to 10. This function allows
 // that to be increased. Set to zero for unlimited.
 EventEmitter.prototype.setMaxListeners = function setMaxListeners(n) {
-  if (typeof n !== 'number' || n < 0 || isNaN(n)) {
+  if (typeof n !== 'number' || n < 0 || Number.isNaN(n)) {
     const errors = lazyErrors();
     throw new errors.TypeError('ERR_OUT_OF_RANGE', 'n');
   }
@@ -115,34 +103,19 @@ EventEmitter.prototype.emit = function emit(type, ...args) {
   else if (!doError)
     return false;
 
-  const domain = this.domain;
-
   // If there is no 'error' event listener then throw.
   if (doError) {
     let er;
     if (args.length > 0)
       er = args[0];
-    if (domain !== null && domain !== undefined) {
-      if (!er) {
-        const errors = lazyErrors();
-        er = new errors.Error('ERR_UNHANDLED_ERROR');
-      }
-      if (typeof er === 'object' && er !== null) {
-        er.domainEmitter = this;
-        er.domain = domain;
-        er.domainThrown = false;
-      }
-      domain.emit('error', er);
-    } else if (er instanceof Error) {
+    if (er instanceof Error) {
       throw er; // Unhandled 'error' event
-    } else {
-      // At least give some kind of context to the user
-      const errors = lazyErrors();
-      const err = new errors.Error('ERR_UNHANDLED_ERROR', er);
-      err.context = er;
-      throw err;
     }
-    return false;
+    // At least give some kind of context to the user
+    const errors = lazyErrors();
+    const err = new errors.Error('ERR_UNHANDLED_ERROR', er);
+    err.context = er;
+    throw err;
   }
 
   const handler = events[type];
@@ -150,24 +123,15 @@ EventEmitter.prototype.emit = function emit(type, ...args) {
   if (handler === undefined)
     return false;
 
-  let needDomainExit = false;
-  if (domain !== null && domain !== undefined && this !== process) {
-    domain.enter();
-    needDomainExit = true;
-  }
-
   if (typeof handler === 'function') {
-    handler.apply(this, args);
+    Reflect.apply(handler, this, args);
   } else {
     const len = handler.length;
     const listeners = arrayClone(handler, len);
     for (var i = 0; i < len; ++i)
-      listeners[i].apply(this, args);
+      Reflect.apply(listeners[i], this, args);
   }
 
-  if (needDomainExit)
-    domain.exit();
-
   return true;
 };
 
@@ -252,7 +216,7 @@ function onceWrapper(...args) {
   if (!this.fired) {
     this.target.removeListener(this.type, this.wrapFn);
     this.fired = true;
-    this.listener.apply(this.target, args);
+    Reflect.apply(this.listener, this.target, args);
   }
 }
 
