@@ -11,16 +11,17 @@ const async_wrap = process.binding('async_wrap');
  * the various asynchronous states of the application. These are:
  *  kExecutionAsyncId: The async_id assigned to the resource responsible for the
  *    current execution stack.
- *  kTriggerAsyncId: The trigger_async_id of the resource responsible for
- *    the current execution stack.
+ *  kTriggerAsyncId: The async_id of the resource that caused (or 'triggered')
+ *    the resource corresponding to the current execution stack.
  *  kAsyncIdCounter: Incremental counter tracking the next assigned async_id.
  *  kDefaultTriggerAsyncId: Written immediately before a resource's constructor
- *    that sets the value of the init()'s triggerAsyncId. The order of
- *    retrieving the triggerAsyncId value is passing directly to the
- *    constructor -> value set in kDefaultTriggerAsyncId -> executionAsyncId of
- *    the current resource.
+ *    that sets the value of the init()'s triggerAsyncId. The precedence order
+ *    of retrieving the triggerAsyncId value is:
+ *    1. the value passed directly to the constructor
+ *    2. value set in kDefaultTriggerAsyncId
+ *    3. executionAsyncId of the current resource.
  *
- * async_ids_fast_stack is a Float64Array that contains part of the async ID
+ * async_ids_stack is a Float64Array that contains part of the async ID
  * stack. Each pushAsyncIds() call adds two doubles to it, and each
  * popAsyncIds() call removes two doubles from it.
  * It has a fixed size, so if that is exceeded, calls to the native
@@ -28,10 +29,10 @@ const async_wrap = process.binding('async_wrap');
  */
 const { async_id_symbol, async_hook_fields, async_id_fields } = async_wrap;
 // Store the pair executionAsyncId and triggerAsyncId in a std::stack on
-// Environment::AsyncHooks::ids_stack_ tracks the resource responsible for the
-// current execution stack. This is unwound as each resource exits. In the case
-// of a fatal exception this stack is emptied after calling each hook's after()
-// callback.
+// Environment::AsyncHooks::async_ids_stack_ tracks the resource responsible for
+// the current execution stack. This is unwound as each resource exits. In the
+// case of a fatal exception this stack is emptied after calling each hook's
+// after() callback.
 const { pushAsyncIds: pushAsyncIds_, popAsyncIds: popAsyncIds_ } = async_wrap;
 // For performance reasons, only track Promises when a hook is enabled.
 const { enablePromiseHook, disablePromiseHook } = async_wrap;
@@ -261,7 +262,7 @@ function getOrSetAsyncId(object) {
 // the user to safeguard this call and make sure it's zero'd out when the
 // constructor is complete.
 function getDefaultTriggerAsyncId() {
-  var defaultTriggerAsyncId = async_id_fields[kDefaultTriggerAsyncId];
+  let defaultTriggerAsyncId = async_id_fields[kDefaultTriggerAsyncId];
   // If defaultTriggerAsyncId isn't set, use the executionAsyncId
   if (defaultTriggerAsyncId < 0)
     defaultTriggerAsyncId = async_id_fields[kExecutionAsyncId];
@@ -275,7 +276,7 @@ function defaultTriggerAsyncIdScope(triggerAsyncId, block, ...args) {
   const oldDefaultTriggerAsyncId = async_id_fields[kDefaultTriggerAsyncId];
   async_id_fields[kDefaultTriggerAsyncId] = triggerAsyncId;
 
-  var ret;
+  let ret;
   try {
     ret = Reflect.apply(block, null, args);
   } finally {
