@@ -409,6 +409,8 @@
     }
   }
 
+  function noop() {}
+
   function setupProcessFatal() {
     const async_wrap = process.binding('async_wrap');
     // Arrays containing hook flags and ids for async_hook calls.
@@ -419,23 +421,15 @@
             kDefaultTriggerAsyncId, kStackLength } = async_wrap.constants;
 
     process._fatalException = function(er) {
-      var caught;
-
       // It's possible that kDefaultTriggerAsyncId was set for a constructor
       // call that threw and was never cleared. So clear it now.
       async_id_fields[kDefaultTriggerAsyncId] = -1;
 
       if (exceptionHandlerState.captureFn !== null) {
         exceptionHandlerState.captureFn(er);
-        caught = true;
-      }
-
-      if (!caught)
-        caught = process.emit('uncaughtException', er);
-
-      // If someone handled it, then great.  otherwise, die in C++ land
-      // since that means that we'll exit the process, emit the 'exit' event
-      if (!caught) {
+      } else if (!process.emit('uncaughtException', er)) {
+        // If someone handled it, then great.  otherwise, die in C++ land
+        // since that means that we'll exit the process, emit the 'exit' event
         try {
           if (!process._exiting) {
             process._exiting = true;
@@ -444,24 +438,25 @@
         } catch (er) {
           // nothing to be done about it at this point.
         }
-
-      } else {
-        // If we handled an error, then make sure any ticks get processed
-        NativeModule.require('timers').setImmediate(process._tickCallback);
-
-        // Emit the after() hooks now that the exception has been handled.
-        if (async_hook_fields[kAfter] > 0) {
-          do {
-            NativeModule.require('internal/async_hooks').emitAfter(
-              async_id_fields[kExecutionAsyncId]);
-          } while (async_hook_fields[kStackLength] > 0);
-        // Or completely empty the id stack.
-        } else {
-          clearAsyncIdStack();
-        }
+        return false;
       }
 
-      return caught;
+      // If we handled an error, then make sure any ticks get processed
+      // by ensuring that the next Immediate cycle isn't empty
+      NativeModule.require('timers').setImmediate(noop);
+
+      // Emit the after() hooks now that the exception has been handled.
+      if (async_hook_fields[kAfter] > 0) {
+        const { emitAfter } = NativeModule.require('internal/async_hooks');
+        do {
+          emitAfter(async_id_fields[kExecutionAsyncId]);
+        } while (async_hook_fields[kStackLength] > 0);
+      // Or completely empty the id stack.
+      } else {
+        clearAsyncIdStack();
+      }
+
+      return true;
     };
   }
 
@@ -634,7 +629,7 @@
   };
 
   NativeModule.wrapper = [
-    '(function (exports, require, module, internalBinding) {',
+    '(function (exports, require, module, internalBinding, process) {',
     '\n});'
   ];
 
@@ -650,7 +645,7 @@
         lineOffset: 0,
         displayErrors: true
       });
-      fn(this.exports, NativeModule.require, this, internalBinding);
+      fn(this.exports, NativeModule.require, this, internalBinding, process);
 
       this.loaded = true;
     } finally {
