@@ -22,6 +22,17 @@
 'use strict';
 
 const errors = require('internal/errors');
+const {
+  CHAR_UPPERCASE_A,
+  CHAR_LOWERCASE_A,
+  CHAR_UPPERCASE_Z,
+  CHAR_LOWERCASE_Z,
+  CHAR_DOT,
+  CHAR_FORWARD_SLASH,
+  CHAR_BACKWARD_SLASH,
+  CHAR_COLON,
+  CHAR_QUESTION_MARK,
+} = require('internal/constants');
 
 function assertPath(path) {
   if (typeof path !== 'string') {
@@ -29,6 +40,15 @@ function assertPath(path) {
   }
 }
 
+function isPathSeparator(code) {
+  return code === CHAR_FORWARD_SLASH || code === CHAR_BACKWARD_SLASH;
+}
+
+function isWindowsDeviceRoot(code) {
+  return code >= CHAR_UPPERCASE_A && code <= CHAR_UPPERCASE_Z ||
+         code >= CHAR_LOWERCASE_A && code <= CHAR_LOWERCASE_Z;
+}
+
 // Resolves . and .. elements in a path with directory names
 function normalizeStringWin32(path, allowAboveRoot) {
   var res = '';
@@ -39,17 +59,18 @@ function normalizeStringWin32(path, allowAboveRoot) {
   for (var i = 0; i <= path.length; ++i) {
     if (i < path.length)
       code = path.charCodeAt(i);
-    else if (code === 47/*/*/ || code === 92/*\*/)
+    else if (isPathSeparator(code))
       break;
     else
-      code = 47/*/*/;
-    if (code === 47/*/*/ || code === 92/*\*/) {
+      code = CHAR_FORWARD_SLASH;
+
+    if (isPathSeparator(code)) {
       if (lastSlash === i - 1 || dots === 1) {
         // NOOP
       } else if (lastSlash !== i - 1 && dots === 2) {
         if (res.length < 2 || lastSegmentLength !== 2 ||
-            res.charCodeAt(res.length - 1) !== 46/*.*/ ||
-            res.charCodeAt(res.length - 2) !== 46/*.*/) {
+            res.charCodeAt(res.length - 1) !== CHAR_DOT ||
+            res.charCodeAt(res.length - 2) !== CHAR_DOT) {
           if (res.length > 2) {
             const lastSlashIndex = res.lastIndexOf('\\');
             if (lastSlashIndex !== res.length - 1) {
@@ -88,7 +109,7 @@ function normalizeStringWin32(path, allowAboveRoot) {
       }
       lastSlash = i;
       dots = 0;
-    } else if (code === 46/*.*/ && dots !== -1) {
+    } else if (code === CHAR_DOT && dots !== -1) {
       ++dots;
     } else {
       dots = -1;
@@ -107,17 +128,17 @@ function normalizeStringPosix(path, allowAboveRoot) {
   for (var i = 0; i <= path.length; ++i) {
     if (i < path.length)
       code = path.charCodeAt(i);
-    else if (code === 47/*/*/)
+    else if (code === CHAR_FORWARD_SLASH)
       break;
     else
-      code = 47/*/*/;
-    if (code === 47/*/*/) {
+      code = CHAR_FORWARD_SLASH;
+    if (code === CHAR_FORWARD_SLASH) {
       if (lastSlash === i - 1 || dots === 1) {
         // NOOP
       } else if (lastSlash !== i - 1 && dots === 2) {
         if (res.length < 2 || lastSegmentLength !== 2 ||
-            res.charCodeAt(res.length - 1) !== 46/*.*/ ||
-            res.charCodeAt(res.length - 2) !== 46/*.*/) {
+            res.charCodeAt(res.length - 1) !== CHAR_DOT ||
+            res.charCodeAt(res.length - 2) !== CHAR_DOT) {
           if (res.length > 2) {
             const lastSlashIndex = res.lastIndexOf('/');
             if (lastSlashIndex !== res.length - 1) {
@@ -156,7 +177,7 @@ function normalizeStringPosix(path, allowAboveRoot) {
       }
       lastSlash = i;
       dots = 0;
-    } else if (code === 46/*.*/ && dots !== -1) {
+    } else if (code === CHAR_DOT && dots !== -1) {
       ++dots;
     } else {
       dots = -1;
@@ -217,28 +238,26 @@ const win32 = {
 
       var len = path.length;
       var rootEnd = 0;
-      var code = path.charCodeAt(0);
       var device = '';
       var isAbsolute = false;
+      const code = path.charCodeAt(0);
 
       // Try to match a root
       if (len > 1) {
-        if (code === 47/*/*/ || code === 92/*\*/) {
+        if (isPathSeparator(code)) {
           // Possible UNC root
 
           // If we started with a separator, we know we at least have an
           // absolute path of some kind (UNC or otherwise)
           isAbsolute = true;
 
-          code = path.charCodeAt(1);
-          if (code === 47/*/*/ || code === 92/*\*/) {
+          if (isPathSeparator(path.charCodeAt(1))) {
             // Matched double path separator at beginning
             var j = 2;
             var last = j;
             // Match 1 or more non-path separators
             for (; j < len; ++j) {
-              code = path.charCodeAt(j);
-              if (code === 47/*/*/ || code === 92/*\*/)
+              if (isPathSeparator(path.charCodeAt(j)))
                 break;
             }
             if (j < len && j !== last) {
@@ -247,8 +266,7 @@ const win32 = {
               last = j;
               // Match 1 or more path separators
               for (; j < len; ++j) {
-                code = path.charCodeAt(j);
-                if (code !== 47/*/*/ && code !== 92/*\*/)
+                if (!isPathSeparator(path.charCodeAt(j)))
                   break;
               }
               if (j < len && j !== last) {
@@ -256,8 +274,7 @@ const win32 = {
                 last = j;
                 // Match 1 or more non-path separators
                 for (; j < len; ++j) {
-                  code = path.charCodeAt(j);
-                  if (code === 47/*/*/ || code === 92/*\*/)
+                  if (isPathSeparator(path.charCodeAt(j)))
                     break;
                 }
                 if (j === len) {
@@ -276,16 +293,14 @@ const win32 = {
           } else {
             rootEnd = 1;
           }
-        } else if ((code >= 65/*A*/ && code <= 90/*Z*/) ||
-                   (code >= 97/*a*/ && code <= 122/*z*/)) {
+        } else if (isWindowsDeviceRoot(code)) {
           // Possible device root
 
-          if (path.charCodeAt(1) === 58/*:*/) {
+          if (path.charCodeAt(1) === CHAR_COLON) {
             device = path.slice(0, 2);
             rootEnd = 2;
             if (len > 2) {
-              code = path.charCodeAt(2);
-              if (code === 47/*/*/ || code === 92/*\*/) {
+              if (isPathSeparator(path.charCodeAt(2))) {
                 // Treat separator following drive name as an absolute path
                 // indicator
                 isAbsolute = true;
@@ -294,7 +309,7 @@ const win32 = {
             }
           }
         }
-      } else if (code === 47/*/*/ || code === 92/*\*/) {
+      } else if (isPathSeparator(code)) {
         // `path` contains just a path separator
         rootEnd = 1;
         isAbsolute = true;
@@ -337,28 +352,26 @@ const win32 = {
     if (len === 0)
       return '.';
     var rootEnd = 0;
-    var code = path.charCodeAt(0);
     var device;
     var isAbsolute = false;
+    const code = path.charCodeAt(0);
 
     // Try to match a root
     if (len > 1) {
-      if (code === 47/*/*/ || code === 92/*\*/) {
+      if (isPathSeparator(code)) {
         // Possible UNC root
 
         // If we started with a separator, we know we at least have an absolute
         // path of some kind (UNC or otherwise)
         isAbsolute = true;
 
-        code = path.charCodeAt(1);
-        if (code === 47/*/*/ || code === 92/*\*/) {
+        if (isPathSeparator(path.charCodeAt(1))) {
           // Matched double path separator at beginning
           var j = 2;
           var last = j;
           // Match 1 or more non-path separators
           for (; j < len; ++j) {
-            code = path.charCodeAt(j);
-            if (code === 47/*/*/ || code === 92/*\*/)
+            if (isPathSeparator(path.charCodeAt(j)))
               break;
           }
           if (j < len && j !== last) {
@@ -367,8 +380,7 @@ const win32 = {
             last = j;
             // Match 1 or more path separators
             for (; j < len; ++j) {
-              code = path.charCodeAt(j);
-              if (code !== 47/*/*/ && code !== 92/*\*/)
+              if (!isPathSeparator(path.charCodeAt(j)))
                 break;
             }
             if (j < len && j !== last) {
@@ -376,8 +388,7 @@ const win32 = {
               last = j;
               // Match 1 or more non-path separators
               for (; j < len; ++j) {
-                code = path.charCodeAt(j);
-                if (code === 47/*/*/ || code === 92/*\*/)
+                if (isPathSeparator(path.charCodeAt(j)))
                   break;
               }
               if (j === len) {
@@ -397,16 +408,14 @@ const win32 = {
         } else {
           rootEnd = 1;
         }
-      } else if ((code >= 65/*A*/ && code <= 90/*Z*/) ||
-                 (code >= 97/*a*/ && code <= 122/*z*/)) {
+      } else if (isWindowsDeviceRoot(code)) {
         // Possible device root
 
-        if (path.charCodeAt(1) === 58/*:*/) {
+        if (path.charCodeAt(1) === CHAR_COLON) {
           device = path.slice(0, 2);
           rootEnd = 2;
           if (len > 2) {
-            code = path.charCodeAt(2);
-            if (code === 47/*/*/ || code === 92/*\*/) {
+            if (isPathSeparator(path.charCodeAt(2))) {
               // Treat separator following drive name as an absolute path
               // indicator
               isAbsolute = true;
@@ -415,14 +424,12 @@ const win32 = {
           }
         }
       }
-    } else if (code === 47/*/*/ || code === 92/*\*/) {
+    } else if (isPathSeparator(code)) {
       // `path` contains just a path separator, exit early to avoid unnecessary
       // work
       return '\\';
     }
 
-    code = path.charCodeAt(len - 1);
-    var trailingSeparator = (code === 47/*/*/ || code === 92/*\*/);
     var tail;
     if (rootEnd < len)
       tail = normalizeStringWin32(path.slice(rootEnd), !isAbsolute);
@@ -430,7 +437,7 @@ const win32 = {
       tail = '';
     if (tail.length === 0 && !isAbsolute)
       tail = '.';
-    if (tail.length > 0 && trailingSeparator)
+    if (tail.length > 0 && isPathSeparator(path.charCodeAt(len - 1)))
       tail += '\\';
     if (device === undefined) {
       if (isAbsolute) {
@@ -461,16 +468,15 @@ const win32 = {
     const len = path.length;
     if (len === 0)
       return false;
-    var code = path.charCodeAt(0);
-    if (code === 47/*/*/ || code === 92/*\*/) {
+
+    const code = path.charCodeAt(0);
+    if (isPathSeparator(code)) {
       return true;
-    } else if ((code >= 65/*A*/ && code <= 90/*Z*/) ||
-               (code >= 97/*a*/ && code <= 122/*z*/)) {
+    } else if (isWindowsDeviceRoot(code)) {
       // Possible device root
 
-      if (len > 2 && path.charCodeAt(1) === 58/*:*/) {
-        code = path.charCodeAt(2);
-        if (code === 47/*/*/ || code === 92/*\*/)
+      if (len > 2 && path.charCodeAt(1) === CHAR_COLON) {
+        if (isPathSeparator(path.charCodeAt(2)))
           return true;
       }
     }
@@ -513,17 +519,14 @@ const win32 = {
     //   path.join('//server', 'share') -> '\\\\server\\share\\')
     var needsReplace = true;
     var slashCount = 0;
-    var code = firstPart.charCodeAt(0);
-    if (code === 47/*/*/ || code === 92/*\*/) {
+    if (isPathSeparator(firstPart.charCodeAt(0))) {
       ++slashCount;
       const firstLen = firstPart.length;
       if (firstLen > 1) {
-        code = firstPart.charCodeAt(1);
-        if (code === 47/*/*/ || code === 92/*\*/) {
+        if (isPathSeparator(firstPart.charCodeAt(1))) {
           ++slashCount;
           if (firstLen > 2) {
-            code = firstPart.charCodeAt(2);
-            if (code === 47/*/*/ || code === 92/*\*/)
+            if (isPathSeparator(firstPart.charCodeAt(2)))
               ++slashCount;
             else {
               // We matched a UNC path in the first part
@@ -536,8 +539,7 @@ const win32 = {
     if (needsReplace) {
       // Find any more consecutive slashes we need to replace
       for (; slashCount < joined.length; ++slashCount) {
-        code = joined.charCodeAt(slashCount);
-        if (code !== 47/*/*/ && code !== 92/*\*/)
+        if (!isPathSeparator(joined.charCodeAt(slashCount)))
           break;
       }
 
@@ -576,13 +578,13 @@ const win32 = {
     // Trim any leading backslashes
     var fromStart = 0;
     for (; fromStart < from.length; ++fromStart) {
-      if (from.charCodeAt(fromStart) !== 92/*\*/)
+      if (from.charCodeAt(fromStart) !== CHAR_BACKWARD_SLASH)
         break;
     }
     // Trim trailing backslashes (applicable to UNC paths only)
     var fromEnd = from.length;
     for (; fromEnd - 1 > fromStart; --fromEnd) {
-      if (from.charCodeAt(fromEnd - 1) !== 92/*\*/)
+      if (from.charCodeAt(fromEnd - 1) !== CHAR_BACKWARD_SLASH)
         break;
     }
     var fromLen = (fromEnd - fromStart);
@@ -590,13 +592,13 @@ const win32 = {
     // Trim any leading backslashes
     var toStart = 0;
     for (; toStart < to.length; ++toStart) {
-      if (to.charCodeAt(toStart) !== 92/*\*/)
+      if (to.charCodeAt(toStart) !== CHAR_BACKWARD_SLASH)
         break;
     }
     // Trim trailing backslashes (applicable to UNC paths only)
     var toEnd = to.length;
     for (; toEnd - 1 > toStart; --toEnd) {
-      if (to.charCodeAt(toEnd - 1) !== 92/*\*/)
+      if (to.charCodeAt(toEnd - 1) !== CHAR_BACKWARD_SLASH)
         break;
     }
     var toLen = (toEnd - toStart);
@@ -608,7 +610,7 @@ const win32 = {
     for (; i <= length; ++i) {
       if (i === length) {
         if (toLen > length) {
-          if (to.charCodeAt(toStart + i) === 92/*\*/) {
+          if (to.charCodeAt(toStart + i) === CHAR_BACKWARD_SLASH) {
             // We get here if `from` is the exact base path for `to`.
             // For example: from='C:\\foo\\bar'; to='C:\\foo\\bar\\baz'
             return toOrig.slice(toStart + i + 1);
@@ -619,7 +621,7 @@ const win32 = {
           }
         }
         if (fromLen > length) {
-          if (from.charCodeAt(fromStart + i) === 92/*\*/) {
+          if (from.charCodeAt(fromStart + i) === CHAR_BACKWARD_SLASH) {
             // We get here if `to` is the exact base path for `from`.
             // For example: from='C:\\foo\\bar'; to='C:\\foo'
             lastCommonSep = i;
@@ -635,7 +637,7 @@ const win32 = {
       var toCode = to.charCodeAt(toStart + i);
       if (fromCode !== toCode)
         break;
-      else if (fromCode === 92/*\*/)
+      else if (fromCode === CHAR_BACKWARD_SLASH)
         lastCommonSep = i;
     }
 
@@ -651,7 +653,7 @@ const win32 = {
     // Generate the relative path based on the path difference between `to` and
     // `from`
     for (i = fromStart + lastCommonSep + 1; i <= fromEnd; ++i) {
-      if (i === fromEnd || from.charCodeAt(i) === 92/*\*/) {
+      if (i === fromEnd || from.charCodeAt(i) === CHAR_BACKWARD_SLASH) {
         if (out.length === 0)
           out += '..';
         else
@@ -665,7 +667,7 @@ const win32 = {
       return out + toOrig.slice(toStart + lastCommonSep, toEnd);
     else {
       toStart += lastCommonSep;
-      if (toOrig.charCodeAt(toStart) === 92/*\*/)
+      if (toOrig.charCodeAt(toStart) === CHAR_BACKWARD_SLASH)
         ++toStart;
       return toOrig.slice(toStart, toEnd);
     }
@@ -684,23 +686,21 @@ const win32 = {
     const resolvedPath = win32.resolve(path);
 
     if (resolvedPath.length >= 3) {
-      var code = resolvedPath.charCodeAt(0);
-      if (code === 92/*\*/) {
+      if (resolvedPath.charCodeAt(0) === CHAR_BACKWARD_SLASH) {
         // Possible UNC root
 
-        if (resolvedPath.charCodeAt(1) === 92/*\*/) {
-          code = resolvedPath.charCodeAt(2);
-          if (code !== 63/*?*/ && code !== 46/*.*/) {
+        if (resolvedPath.charCodeAt(1) === CHAR_BACKWARD_SLASH) {
+          const code = resolvedPath.charCodeAt(2);
+          if (code !== CHAR_QUESTION_MARK && code !== CHAR_DOT) {
             // Matched non-long UNC root, convert the path to a long UNC path
             return '\\\\?\\UNC\\' + resolvedPath.slice(2);
           }
         }
-      } else if ((code >= 65/*A*/ && code <= 90/*Z*/) ||
-                 (code >= 97/*a*/ && code <= 122/*z*/)) {
+      } else if (isWindowsDeviceRoot(resolvedPath.charCodeAt(0))) {
         // Possible device root
 
-        if (resolvedPath.charCodeAt(1) === 58/*:*/ &&
-            resolvedPath.charCodeAt(2) === 92/*\*/) {
+        if (resolvedPath.charCodeAt(1) === CHAR_COLON &&
+            resolvedPath.charCodeAt(2) === CHAR_BACKWARD_SLASH) {
           // Matched device root, convert the path to a long UNC path
           return '\\\\?\\' + resolvedPath;
         }
@@ -719,24 +719,22 @@ const win32 = {
     var end = -1;
     var matchedSlash = true;
     var offset = 0;
-    var code = path.charCodeAt(0);
+    const code = path.charCodeAt(0);
 
     // Try to match a root
     if (len > 1) {
-      if (code === 47/*/*/ || code === 92/*\*/) {
+      if (isPathSeparator(code)) {
         // Possible UNC root
 
         rootEnd = offset = 1;
 
-        code = path.charCodeAt(1);
-        if (code === 47/*/*/ || code === 92/*\*/) {
+        if (isPathSeparator(path.charCodeAt(1))) {
           // Matched double path separator at beginning
           var j = 2;
           var last = j;
           // Match 1 or more non-path separators
           for (; j < len; ++j) {
-            code = path.charCodeAt(j);
-            if (code === 47/*/*/ || code === 92/*\*/)
+            if (isPathSeparator(path.charCodeAt(j)))
               break;
           }
           if (j < len && j !== last) {
@@ -744,8 +742,7 @@ const win32 = {
             last = j;
             // Match 1 or more path separators
             for (; j < len; ++j) {
-              code = path.charCodeAt(j);
-              if (code !== 47/*/*/ && code !== 92/*\*/)
+              if (!isPathSeparator(path.charCodeAt(j)))
                 break;
             }
             if (j < len && j !== last) {
@@ -753,8 +750,7 @@ const win32 = {
               last = j;
               // Match 1 or more non-path separators
               for (; j < len; ++j) {
-                code = path.charCodeAt(j);
-                if (code === 47/*/*/ || code === 92/*\*/)
+                if (isPathSeparator(path.charCodeAt(j)))
                   break;
               }
               if (j === len) {
@@ -771,28 +767,25 @@ const win32 = {
             }
           }
         }
-      } else if ((code >= 65/*A*/ && code <= 90/*Z*/) ||
-                 (code >= 97/*a*/ && code <= 122/*z*/)) {
+      } else if (isWindowsDeviceRoot(code)) {
         // Possible device root
 
-        if (path.charCodeAt(1) === 58/*:*/) {
+        if (path.charCodeAt(1) === CHAR_COLON) {
           rootEnd = offset = 2;
           if (len > 2) {
-            code = path.charCodeAt(2);
-            if (code === 47/*/*/ || code === 92/*\*/)
+            if (isPathSeparator(path.charCodeAt(2)))
               rootEnd = offset = 3;
           }
         }
       }
-    } else if (code === 47/*/*/ || code === 92/*\*/) {
+    } else if (isPathSeparator(code)) {
       // `path` contains just a path separator, exit early to avoid
       // unnecessary work
       return path;
     }
 
     for (var i = len - 1; i >= offset; --i) {
-      code = path.charCodeAt(i);
-      if (code === 47/*/*/ || code === 92/*\*/) {
+      if (isPathSeparator(path.charCodeAt(i))) {
         if (!matchedSlash) {
           end = i;
           break;
@@ -827,9 +820,8 @@ const win32 = {
     // disregarded
     if (path.length >= 2) {
       const drive = path.charCodeAt(0);
-      if ((drive >= 65/*A*/ && drive <= 90/*Z*/) ||
-          (drive >= 97/*a*/ && drive <= 122/*z*/)) {
-        if (path.charCodeAt(1) === 58/*:*/)
+      if (isWindowsDeviceRoot(drive)) {
+        if (path.charCodeAt(1) === CHAR_COLON)
           start = 2;
       }
     }
@@ -841,7 +833,7 @@ const win32 = {
       var firstNonSlashEnd = -1;
       for (i = path.length - 1; i >= start; --i) {
         const code = path.charCodeAt(i);
-        if (code === 47/*/*/ || code === 92/*\*/) {
+        if (isPathSeparator(code)) {
           // If we reached a path separator that was not part of a set of path
           // separators at the end of the string, stop now
           if (!matchedSlash) {
@@ -880,8 +872,7 @@ const win32 = {
       return path.slice(start, end);
     } else {
       for (i = path.length - 1; i >= start; --i) {
-        const code = path.charCodeAt(i);
-        if (code === 47/*/*/ || code === 92/*\*/) {
+        if (isPathSeparator(path.charCodeAt(i))) {
           // If we reached a path separator that was not part of a set of path
           // separators at the end of the string, stop now
           if (!matchedSlash) {
@@ -917,18 +908,16 @@ const win32 = {
     // Check for a drive letter prefix so as not to mistake the following
     // path separator as an extra separator at the end of the path that can be
     // disregarded
-    if (path.length >= 2) {
-      const code = path.charCodeAt(0);
-      if (path.charCodeAt(1) === 58/*:*/ &&
-          ((code >= 65/*A*/ && code <= 90/*Z*/) ||
-           (code >= 97/*a*/ && code <= 122/*z*/))) {
-        start = startPart = 2;
-      }
+
+    if (path.length >= 2 &&
+        path.charCodeAt(1) === CHAR_COLON &&
+        isWindowsDeviceRoot(path.charCodeAt(0))) {
+      start = startPart = 2;
     }
 
     for (var i = path.length - 1; i >= start; --i) {
       const code = path.charCodeAt(i);
-      if (code === 47/*/*/ || code === 92/*\*/) {
+      if (isPathSeparator(code)) {
         // If we reached a path separator that was not part of a set of path
         // separators at the end of the string, stop now
         if (!matchedSlash) {
@@ -943,7 +932,7 @@ const win32 = {
         matchedSlash = false;
         end = i + 1;
       }
-      if (code === 46/*.*/) {
+      if (code === CHAR_DOT) {
         // If this is our first dot, mark it as the start of our extension
         if (startDot === -1)
           startDot = i;
@@ -988,23 +977,21 @@ const win32 = {
 
     var len = path.length;
     var rootEnd = 0;
-    var code = path.charCodeAt(0);
+    let code = path.charCodeAt(0);
 
     // Try to match a root
     if (len > 1) {
-      if (code === 47/*/*/ || code === 92/*\*/) {
+      if (isPathSeparator(code)) {
         // Possible UNC root
 
-        code = path.charCodeAt(1);
         rootEnd = 1;
-        if (code === 47/*/*/ || code === 92/*\*/) {
+        if (isPathSeparator(path.charCodeAt(1))) {
           // Matched double path separator at beginning
           var j = 2;
           var last = j;
           // Match 1 or more non-path separators
           for (; j < len; ++j) {
-            code = path.charCodeAt(j);
-            if (code === 47/*/*/ || code === 92/*\*/)
+            if (isPathSeparator(path.charCodeAt(j)))
               break;
           }
           if (j < len && j !== last) {
@@ -1012,8 +999,7 @@ const win32 = {
             last = j;
             // Match 1 or more path separators
             for (; j < len; ++j) {
-              code = path.charCodeAt(j);
-              if (code !== 47/*/*/ && code !== 92/*\*/)
+              if (!isPathSeparator(path.charCodeAt(j)))
                 break;
             }
             if (j < len && j !== last) {
@@ -1021,8 +1007,7 @@ const win32 = {
               last = j;
               // Match 1 or more non-path separators
               for (; j < len; ++j) {
-                code = path.charCodeAt(j);
-                if (code === 47/*/*/ || code === 92/*\*/)
+                if (isPathSeparator(path.charCodeAt(j)))
                   break;
               }
               if (j === len) {
@@ -1037,15 +1022,13 @@ const win32 = {
             }
           }
         }
-      } else if ((code >= 65/*A*/ && code <= 90/*Z*/) ||
-                 (code >= 97/*a*/ && code <= 122/*z*/)) {
+      } else if (isWindowsDeviceRoot(code)) {
         // Possible device root
 
-        if (path.charCodeAt(1) === 58/*:*/) {
+        if (path.charCodeAt(1) === CHAR_COLON) {
           rootEnd = 2;
           if (len > 2) {
-            code = path.charCodeAt(2);
-            if (code === 47/*/*/ || code === 92/*\*/) {
+            if (isPathSeparator(path.charCodeAt(2))) {
               if (len === 3) {
                 // `path` contains just a drive root, exit early to avoid
                 // unnecessary work
@@ -1062,7 +1045,7 @@ const win32 = {
           }
         }
       }
-    } else if (code === 47/*/*/ || code === 92/*\*/) {
+    } else if (isPathSeparator(code)) {
       // `path` contains just a path separator, exit early to avoid
       // unnecessary work
       ret.root = ret.dir = path;
@@ -1085,7 +1068,7 @@ const win32 = {
     // Get non-dir info
     for (; i >= rootEnd; --i) {
       code = path.charCodeAt(i);
-      if (code === 47/*/*/ || code === 92/*\*/) {
+      if (isPathSeparator(code)) {
         // If we reached a path separator that was not part of a set of path
         // separators at the end of the string, stop now
         if (!matchedSlash) {
@@ -1100,7 +1083,7 @@ const win32 = {
         matchedSlash = false;
         end = i + 1;
       }
-      if (code === 46/*.*/) {
+      if (code === CHAR_DOT) {
         // If this is our first dot, mark it as the start of our extension
         if (startDot === -1)
           startDot = i;
@@ -1174,7 +1157,7 @@ const posix = {
       }
 
       resolvedPath = path + '/' + resolvedPath;
-      resolvedAbsolute = path.charCodeAt(0) === 47/*/*/;
+      resolvedAbsolute = path.charCodeAt(0) === CHAR_FORWARD_SLASH;
     }
 
     // At this point the path should be resolved to a full absolute path, but
@@ -1202,8 +1185,9 @@ const posix = {
     if (path.length === 0)
       return '.';
 
-    const isAbsolute = path.charCodeAt(0) === 47/*/*/;
-    const trailingSeparator = path.charCodeAt(path.length - 1) === 47/*/*/;
+    const isAbsolute = path.charCodeAt(0) === CHAR_FORWARD_SLASH;
+    const trailingSeparator =
+      path.charCodeAt(path.length - 1) === CHAR_FORWARD_SLASH;
 
     // Normalize the path
     path = normalizeStringPosix(path, !isAbsolute);
@@ -1221,7 +1205,7 @@ const posix = {
 
   isAbsolute: function isAbsolute(path) {
     assertPath(path);
-    return path.length > 0 && path.charCodeAt(0) === 47/*/*/;
+    return path.length > 0 && path.charCodeAt(0) === CHAR_FORWARD_SLASH;
   },
 
 
@@ -1261,7 +1245,7 @@ const posix = {
     // Trim any leading backslashes
     var fromStart = 1;
     for (; fromStart < from.length; ++fromStart) {
-      if (from.charCodeAt(fromStart) !== 47/*/*/)
+      if (from.charCodeAt(fromStart) !== CHAR_FORWARD_SLASH)
         break;
     }
     var fromEnd = from.length;
@@ -1270,7 +1254,7 @@ const posix = {
     // Trim any leading backslashes
     var toStart = 1;
     for (; toStart < to.length; ++toStart) {
-      if (to.charCodeAt(toStart) !== 47/*/*/)
+      if (to.charCodeAt(toStart) !== CHAR_FORWARD_SLASH)
         break;
     }
     var toEnd = to.length;
@@ -1283,7 +1267,7 @@ const posix = {
     for (; i <= length; ++i) {
       if (i === length) {
         if (toLen > length) {
-          if (to.charCodeAt(toStart + i) === 47/*/*/) {
+          if (to.charCodeAt(toStart + i) === CHAR_FORWARD_SLASH) {
             // We get here if `from` is the exact base path for `to`.
             // For example: from='/foo/bar'; to='/foo/bar/baz'
             return to.slice(toStart + i + 1);
@@ -1293,7 +1277,7 @@ const posix = {
             return to.slice(toStart + i);
           }
         } else if (fromLen > length) {
-          if (from.charCodeAt(fromStart + i) === 47/*/*/) {
+          if (from.charCodeAt(fromStart + i) === CHAR_FORWARD_SLASH) {
             // We get here if `to` is the exact base path for `from`.
             // For example: from='/foo/bar/baz'; to='/foo/bar'
             lastCommonSep = i;
@@ -1309,7 +1293,7 @@ const posix = {
       var toCode = to.charCodeAt(toStart + i);
       if (fromCode !== toCode)
         break;
-      else if (fromCode === 47/*/*/)
+      else if (fromCode === CHAR_FORWARD_SLASH)
         lastCommonSep = i;
     }
 
@@ -1317,7 +1301,7 @@ const posix = {
     // Generate the relative path based on the path difference between `to`
     // and `from`
     for (i = fromStart + lastCommonSep + 1; i <= fromEnd; ++i) {
-      if (i === fromEnd || from.charCodeAt(i) === 47/*/*/) {
+      if (i === fromEnd || from.charCodeAt(i) === CHAR_FORWARD_SLASH) {
         if (out.length === 0)
           out += '..';
         else
@@ -1331,7 +1315,7 @@ const posix = {
       return out + to.slice(toStart + lastCommonSep);
     else {
       toStart += lastCommonSep;
-      if (to.charCodeAt(toStart) === 47/*/*/)
+      if (to.charCodeAt(toStart) === CHAR_FORWARD_SLASH)
         ++toStart;
       return to.slice(toStart);
     }
@@ -1347,13 +1331,11 @@ const posix = {
     assertPath(path);
     if (path.length === 0)
       return '.';
-    var code = path.charCodeAt(0);
-    var hasRoot = (code === 47/*/*/);
+    const hasRoot = path.charCodeAt(0) === CHAR_FORWARD_SLASH;
     var end = -1;
     var matchedSlash = true;
     for (var i = path.length - 1; i >= 1; --i) {
-      code = path.charCodeAt(i);
-      if (code === 47/*/*/) {
+      if (path.charCodeAt(i) === CHAR_FORWARD_SLASH) {
         if (!matchedSlash) {
           end = i;
           break;
@@ -1389,7 +1371,7 @@ const posix = {
       var firstNonSlashEnd = -1;
       for (i = path.length - 1; i >= 0; --i) {
         const code = path.charCodeAt(i);
-        if (code === 47/*/*/) {
+        if (code === CHAR_FORWARD_SLASH) {
           // If we reached a path separator that was not part of a set of path
           // separators at the end of the string, stop now
           if (!matchedSlash) {
@@ -1428,7 +1410,7 @@ const posix = {
       return path.slice(start, end);
     } else {
       for (i = path.length - 1; i >= 0; --i) {
-        if (path.charCodeAt(i) === 47/*/*/) {
+        if (path.charCodeAt(i) === CHAR_FORWARD_SLASH) {
           // If we reached a path separator that was not part of a set of path
           // separators at the end of the string, stop now
           if (!matchedSlash) {
@@ -1461,7 +1443,7 @@ const posix = {
     var preDotState = 0;
     for (var i = path.length - 1; i >= 0; --i) {
       const code = path.charCodeAt(i);
-      if (code === 47/*/*/) {
+      if (code === CHAR_FORWARD_SLASH) {
         // If we reached a path separator that was not part of a set of path
         // separators at the end of the string, stop now
         if (!matchedSlash) {
@@ -1476,7 +1458,7 @@ const posix = {
         matchedSlash = false;
         end = i + 1;
       }
-      if (code === 46/*.*/) {
+      if (code === CHAR_DOT) {
         // If this is our first dot, mark it as the start of our extension
         if (startDot === -1)
           startDot = i;
@@ -1518,8 +1500,7 @@ const posix = {
     var ret = { root: '', dir: '', base: '', ext: '', name: '' };
     if (path.length === 0)
       return ret;
-    var code = path.charCodeAt(0);
-    var isAbsolute = (code === 47/*/*/);
+    var isAbsolute = path.charCodeAt(0) === CHAR_FORWARD_SLASH;
     var start;
     if (isAbsolute) {
       ret.root = '/';
@@ -1539,8 +1520,8 @@ const posix = {
 
     // Get non-dir info
     for (; i >= start; --i) {
-      code = path.charCodeAt(i);
-      if (code === 47/*/*/) {
+      const code = path.charCodeAt(i);
+      if (code === CHAR_FORWARD_SLASH) {
         // If we reached a path separator that was not part of a set of path
         // separators at the end of the string, stop now
         if (!matchedSlash) {
@@ -1555,7 +1536,7 @@ const posix = {
         matchedSlash = false;
         end = i + 1;
       }
-      if (code === 46/*.*/) {
+      if (code === CHAR_DOT) {
         // If this is our first dot, mark it as the start of our extension
         if (startDot === -1)
           startDot = i;
