@@ -650,7 +650,7 @@ REPLServer.prototype.createContext = function() {
   } else {
     sendInspectorCommand((session) => {
       session.post('Runtime.enable');
-      session.on('Runtime.executionContextCreated', ({ params }) => {
+      session.once('Runtime.executionContextCreated', ({ params }) => {
         this[kContextId] = params.context.id;
       });
       context = vm.createContext();
@@ -834,7 +834,6 @@ function complete(line, callback) {
     var flat = new ArrayStream();         // make a new "input" stream
     var magic = new REPLServer('', flat); // make a nested REPL
     replMap.set(magic, replMap.get(this));
-    magic.resetContext();
     flat.run(tmp);                        // eval the flattened code
     // all this is only profitable if the nested REPL
     // does not have a bufferedCommand
