@@ -34,18 +34,22 @@ assert.throws(() => assert.deepStrictEqual(arr, buf),
               re`${arr} deepStrictEqual ${buf}`);
 assert.doesNotThrow(() => assert.deepEqual(arr, buf));
 
-const buf2 = Buffer.from(arr);
-buf2.prop = 1;
+{
+  const buf2 = Buffer.from(arr);
+  buf2.prop = 1;
 
-assert.throws(() => assert.deepStrictEqual(buf2, buf),
-              re`${buf2} deepStrictEqual ${buf}`);
-assert.doesNotThrow(() => assert.deepEqual(buf2, buf));
+  assert.throws(() => assert.deepStrictEqual(buf2, buf),
+                re`${buf2} deepStrictEqual ${buf}`);
+  assert.doesNotThrow(() => assert.deepEqual(buf2, buf));
+}
 
-const arr2 = new Uint8Array([120, 121, 122, 10]);
-arr2.prop = 5;
-assert.throws(() => assert.deepStrictEqual(arr, arr2),
-              re`${arr} deepStrictEqual ${arr2}`);
-assert.doesNotThrow(() => assert.deepEqual(arr, arr2));
+{
+  const arr2 = new Uint8Array([120, 121, 122, 10]);
+  arr2.prop = 5;
+  assert.throws(() => assert.deepStrictEqual(arr, arr2),
+                re`${arr} deepStrictEqual ${arr2}`);
+  assert.doesNotThrow(() => assert.deepEqual(arr, arr2));
+}
 
 const date = new Date('2016');
 
@@ -85,41 +89,43 @@ assert.throws(() => assert.deepStrictEqual(re1, re2),
 
 // For these weird cases, deepEqual should pass (at least for now),
 // but deepStrictEqual should throw.
-const similar = new Set([
-  { 0: '1' },  // Object
-  { 0: 1 },  // Object
-  new String('1'),  // Object
-  ['1'],  // Array
-  [1],  // Array
-  date2,  // Date with this[0] = '1'
-  re2,  // RegExp with this[0] = '1'
-  new Int8Array([1]), // Int8Array
-  new Uint8Array([1]), // Uint8Array
-  new Int16Array([1]), // Int16Array
-  new Uint16Array([1]), // Uint16Array
-  new Int32Array([1]), // Int32Array
-  new Uint32Array([1]), // Uint32Array
-  Buffer.from([1]),
-  // Arguments {'0': '1'} is not here
-  // See https://github.com/nodejs/node-v0.x-archive/pull/7178
-]);
+{
+  const similar = new Set([
+    { 0: '1' },  // Object
+    { 0: 1 },  // Object
+    new String('1'),  // Object
+    ['1'],  // Array
+    [1],  // Array
+    date2,  // Date with this[0] = '1'
+    re2,  // RegExp with this[0] = '1'
+    new Int8Array([1]), // Int8Array
+    new Uint8Array([1]), // Uint8Array
+    new Int16Array([1]), // Int16Array
+    new Uint16Array([1]), // Uint16Array
+    new Int32Array([1]), // Int32Array
+    new Uint32Array([1]), // Uint32Array
+    Buffer.from([1]),
+    // Arguments {'0': '1'} is not here
+    // See https://github.com/nodejs/node-v0.x-archive/pull/7178
+  ]);
 
-for (const a of similar) {
-  for (const b of similar) {
-    if (a !== b) {
-      assert.deepEqual(a, b);
-      assert.throws(() => assert.deepStrictEqual(a, b),
-                    re`${a} deepStrictEqual ${b}`);
+  for (const a of similar) {
+    for (const b of similar) {
+      if (a !== b) {
+        assert.deepEqual(a, b);
+        assert.throws(() => assert.deepStrictEqual(a, b),
+                      re`${a} deepStrictEqual ${b}`);
+      }
     }
   }
 }
 
-assert.throws(
-  () => { assert.deepEqual(new Set([{ a: 0 }]), new Set([{ a: 1 }])); },
-  common.expectsError({
-    code: 'ERR_ASSERTION',
-    message: /^Set { { a: 0 } } deepEqual Set { { a: 1 } }$/
-  }));
+common.expectsError(() => {
+  assert.deepEqual(new Set([{ a: 0 }]), new Set([{ a: 1 }]));
+}, {
+  code: 'ERR_ASSERTION',
+  message: /^Set { { a: 0 } } deepEqual Set { { a: 1 } }$/
+});
 
 function assertDeepAndStrictEqual(a, b) {
   assert.deepEqual(a, b);
@@ -466,6 +472,7 @@ assertOnlyDeepEqual(
   assertDeepAndStrictEqual(m3, m4);
 }
 
+// Handle sparse arrays
 assertDeepAndStrictEqual([1, , , 3], [1, , , 3]);
 assertOnlyDeepEqual([1, , , 3], [1, , , 3, , , ]);
 
@@ -481,6 +488,13 @@ assertOnlyDeepEqual([1, , , 3], [1, , , 3, , , ]);
   assertOnlyDeepEqual(err1, {}, assert.AssertionError);
 }
 
+// Handle NaN
+assert.throws(() => { assert.deepEqual(NaN, NaN); }, assert.AssertionError);
+assert.doesNotThrow(() => { assert.deepStrictEqual(NaN, NaN); });
+assert.doesNotThrow(() => { assert.deepStrictEqual({ a: NaN }, { a: NaN }); });
+assert.doesNotThrow(
+  () => { assert.deepStrictEqual([ 1, 2, NaN, 4 ], [ 1, 2, NaN, 4 ]); });
+
 // Handle boxed primitives
 {
   const boxedString = new String('test');
@@ -500,4 +514,35 @@ assertOnlyDeepEqual([1, , , 3], [1, , , 3, , , ]);
   assertNotDeepOrStrict(boxedSymbol, {});
 }
 
+// Minus zero
+assertOnlyDeepEqual(0, -0);
+assertDeepAndStrictEqual(-0, -0);
+
+// Handle symbols (enumerable only)
+{
+  const symbol1 = Symbol();
+  const obj1 = { [symbol1]: 1 };
+  const obj2 = { [symbol1]: 1 };
+  const obj3 = { [Symbol()]: 1 };
+  // Add a non enumerable symbol as well. It is going to be ignored!
+  Object.defineProperty(obj2, Symbol(), { value: 1 });
+  assertOnlyDeepEqual(obj1, obj3);
+  assertDeepAndStrictEqual(obj1, obj2);
+  // TypedArrays have a fast path. Test for this as well.
+  const a = new Uint8Array(4);
+  const b = new Uint8Array(4);
+  a[symbol1] = true;
+  b[symbol1] = false;
+  assertOnlyDeepEqual(a, b);
+  b[symbol1] = true;
+  assertDeepAndStrictEqual(a, b);
+  // The same as TypedArrays is valid for boxed primitives
+  const boxedStringA = new String('test');
+  const boxedStringB = new String('test');
+  boxedStringA[symbol1] = true;
+  assertOnlyDeepEqual(boxedStringA, boxedStringB);
+  boxedStringA[symbol1] = true;
+  assertDeepAndStrictEqual(a, b);
+}
+
 /* eslint-enable */
