@@ -8,8 +8,9 @@ if (process.config.variables.node_without_node_options)
 const assert = require('assert');
 const exec = require('child_process').execFile;
 
-common.refreshTmpDir();
-process.chdir(common.tmpDir);
+const tmpdir = require('../common/tmpdir');
+tmpdir.refresh();
+process.chdir(tmpdir.path);
 
 expect(`-r ${require.resolve('../fixtures/printA.js')}`, 'A\nB\n');
 expect('--no-deprecation', 'B\n');
@@ -23,6 +24,16 @@ expect('--track-heap-objects', 'B\n');
 expect('--throw-deprecation', 'B\n');
 expect('--zero-fill-buffers', 'B\n');
 expect('--v8-pool-size=10', 'B\n');
+expect('--trace-event-categories node', 'B\n');
+
+if (!common.isWindows) {
+  expect('--perf-basic-prof', 'B\n');
+}
+
+if (common.isLinux && ['arm', 'x64', 'mips'].includes(process.arch)) {
+  // PerfJitLogger is only implemented in Linux.
+  expect('--perf-prof', 'B\n');
+}
 
 if (common.hasCrypto) {
   expect('--use-openssl-ca', 'B\n');
