@@ -1,10 +1,8 @@
 'use strict';
-require('../common');
+const common = require('../common');
 const assert = require('assert');
 const net = require('net');
 
-const expectedError = /^TypeError: "lookup" option should be a function$/;
-
 ['foobar', 1, {}, []].forEach((input) => connectThrows(input));
 
 // Using port 0 as lookup is emitted before connecting.
@@ -15,12 +13,15 @@ function connectThrows(input) {
     lookup: input
   };
 
-  assert.throws(() => {
+  common.expectsError(() => {
     net.connect(opts);
-  }, expectedError);
+  }, {
+    code: 'ERR_INVALID_ARG_TYPE',
+    type: TypeError
+  });
 }
 
-[() => {}].forEach((input) => connectDoesNotThrow(input));
+connectDoesNotThrow(() => {});
 
 function connectDoesNotThrow(input) {
   const opts = {
